from django.conf import settings
from django.conf.urls.i18n import i18n_patterns
from django.conf.urls.static import static
from django.contrib import admin
from django.contrib.auth import views as auth_views
from django.contrib.auth.decorators import login_required
from django.contrib.staticfiles.storage import staticfiles_storage
from django.contrib.staticfiles.urls import staticfiles_urlpatterns
from django.urls import include, path, re_path
from django.views.decorators.cache import cache_control, cache_page, never_cache
from django.views.decorators.csrf import ensure_csrf_cookie
from django.views.generic.base import RedirectView

from . import views
from .decorators import (
    can_edit_map,
    can_view_map,
    login_required_if_not_anonymous_allowed,
    team_members_only,
)
from .utils import decorated_patterns

admin.autodiscover()

urlpatterns = [
    re_path(r"^admin/", admin.site.urls),
    re_path("", include("social_django.urls", namespace="social")),
    re_path(
        r"^agnocomplete/",
        include(("agnocomplete.urls", "agnocomplete"), namespace="agnocomplete"),
    ),
    re_path(r"^m/(?P<pk>\d+)/$", views.MapShortUrl.as_view(), name="map_short_url"),
    re_path(r"^ajax-proxy/$", cache_page(180)(views.ajax_proxy), name="ajax-proxy"),
    re_path(
        r"^change-password/",
        auth_views.PasswordChangeView.as_view(),
        {"template_name": "umap/password_change.html"},
        name="password_change",
    ),
    re_path(
        r"^change-password-done/",
        auth_views.PasswordChangeDoneView.as_view(),
        {"template_name": "umap/password_change_done.html"},
        name="password_change_done",
    ),
    re_path(r"^i18n/", include("django.conf.urls.i18n")),
    re_path(r"^map/oembed/", views.MapOEmbed.as_view(), name="map_oembed"),
    re_path(
        r"^map/(?P<map_id>\d+)/download/",
        can_view_map(views.MapDownload.as_view()),
        name="map_download",
    ),
]

i18n_urls = [
    re_path(r"^login/$", auth_views.LoginView.as_view(), name="login"),
    re_path(
        r"^login/popup/end/$", views.LoginPopupEnd.as_view(), name="login_popup_end"
    ),
    re_path(r"^logout/$", views.logout, name="logout"),
    re_path(
        r"^map/(?P<map_id>\d+)/geojson/$",
        views.MapViewGeoJSON.as_view(),
        name="map_geojson",
    ),
    re_path(
        r"^map/anonymous-edit/(?P<signature>.+)$",
        views.MapAnonymousEditUrl.as_view(),
        name="map_anonymous_edit_url",
    ),
    re_path(
        r"^pictogram/json/$",
        views.PictogramJSONList.as_view(),
        name="pictogram_list_json",
    ),
]
i18n_urls += decorated_patterns(
    [can_view_map, cache_control(must_revalidate=True)],
    path(
        "datalayer/<int:map_id>/<uuid:pk>/",
        views.DataLayerView.as_view(),
        name="datalayer_view",
    ),
    path(
        "datalayer/<int:map_id>/<uuid:pk>/versions/",
        views.DataLayerVersions.as_view(),
        name="datalayer_versions",
    ),
    path(
        "datalayer/<int:map_id>/<uuid:pk>/<str:ref>",
        views.DataLayerVersion.as_view(),
        name="datalayer_version",
    ),
)
i18n_urls += decorated_patterns(
    [can_view_map, ensure_csrf_cookie],
    re_path(
        r"^map/(?P<slug>[-_\w]+)_(?P<map_id>\d+)$", views.MapView.as_view(), name="map"
    ),
)
i18n_urls += decorated_patterns(
    [ensure_csrf_cookie],
    path("map/", views.MapPreview.as_view(), name="map_preview"),
    path("map/new/", views.MapNew.as_view(), name="map_new"),
)
i18n_urls += decorated_patterns(
    [login_required_if_not_anonymous_allowed, never_cache],
    path("map/create/", views.MapCreate.as_view(), name="map_create"),
)
i18n_urls += decorated_patterns(
    [login_required],
    re_path(
        r"^map/(?P<map_id>[\d]+)/star/$",
        views.ToggleMapStarStatus.as_view(),
        name="map_star",
    ),
    path("me", views.user_dashboard, name="user_dashboard"),
    path("me/download", views.user_download, name="user_download"),
    path("me/teams", views.UserTeams.as_view(), name="user_teams"),
    path("team/create/", views.TeamNew.as_view(), name="team_new"),
)

if settings.UMAP_ALLOW_EDIT_PROFILE:
    i18n_urls.append(
        path("me/profile", login_required(views.user_profile), name="user_profile")
    )
i18n_urls += decorated_patterns(
    [login_required, team_members_only],
    path("team/<int:pk>/edit/", views.TeamUpdate.as_view(), name="team_update"),
    path("team/<int:pk>/delete/", views.TeamDelete.as_view(), name="team_delete"),
)
map_urls = [
    re_path(
        r"^map/(?P<map_id>[\d]+)/update/settings/$",
        views.MapUpdate.as_view(),
        name="map_update",
    ),
    re_path(
        r"^map/(?P<map_id>[\d]+)/update/permissions/$",
        views.UpdateMapPermissions.as_view(),
        name="map_update_permissions",
    ),
    re_path(
        r"^map/(?P<map_id>[\d]+)/update/owner/$",
        views.AttachAnonymousMap.as_view(),
        name="map_attach_owner",
    ),
    re_path(
        r"^map/(?P<map_id>[\d]+)/update/delete/$",
        views.MapDelete.as_view(),
        name="map_delete",
    ),
    re_path(
        r"^map/(?P<map_id>[\d]+)/update/clone/$",
        views.MapClone.as_view(),
        name="map_clone",
    ),
    path(
        "map/<int:map_id>/datalayer/create/<uuid:pk>/",
        views.DataLayerCreate.as_view(),
        name="datalayer_create",
    ),
    path(
        "map/<int:map_id>/datalayer/delete/<uuid:pk>/",
        views.DataLayerDelete.as_view(),
        name="datalayer_delete",
    ),
    path(
        "map/<int:map_id>/datalayer/permissions/<uuid:pk>/",
        views.UpdateDataLayerPermissions.as_view(),
        name="datalayer_permissions",
    ),
    path(
        "map/<int:map_id>/ws-token/",
        views.get_websocket_auth_token,
        name="map_websocket_auth_token",
    ),
]
if settings.DEFAULT_FROM_EMAIL:
    map_urls.append(
        re_path(
            r"^map/(?P<map_id>[\d]+)/send-edit-link/$",
            views.SendEditLink.as_view(),
            name="map_send_edit_link",
        )
    )
datalayer_urls = [
    path(
        "map/<int:map_id>/datalayer/update/<uuid:pk>/",
        views.DataLayerUpdate.as_view(),
        name="datalayer_update",
    ),
]
i18n_urls += decorated_patterns([can_edit_map, never_cache], *map_urls)
i18n_urls += decorated_patterns([never_cache], *datalayer_urls)
urlpatterns += i18n_patterns(
    path("", views.home, name="home"),
    path("showcase/", cache_page(24 * 60 * 60)(views.showcase), name="maps_showcase"),
    path("search/", views.search, name="search"),
    path("about/", views.about, name="about"),
    re_path(r"^user/(?P<identifier>.+)/stars/$", views.user_stars, name="user_stars"),
    re_path(r"^user/(?P<identifier>.+)/$", views.user_maps, name="user_maps"),
    path("team/<int:pk>/", views.TeamMaps.as_view(), name="team_maps"),
    re_path(r"", include(i18n_urls)),
)
urlpatterns += (
    path("stats/", cache_page(60 * 60)(views.stats), name="stats"),
    path(
        "favicon.ico",
        cache_control(max_age=60 * 60 * 24, immutable=True, public=True)(
            RedirectView.as_view(
                url=staticfiles_storage.url("umap/favicons/favicon.ico")
            )
        ),
    ),
    path(
        "manifest.webmanifest",
        cache_control(max_age=60 * 60 * 24, immutable=True, public=True)(
            views.webmanifest
        ),
    ),
)

if settings.DEBUG and settings.MEDIA_ROOT:
    urlpatterns += static(settings.MEDIA_URL, document_root=settings.MEDIA_ROOT)
urlpatterns += staticfiles_urlpatterns()



# Generated by Django 5.1.4 on 2025-01-29 18:06

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("umap", "0025_alter_datalayer_geojson"),
    ]

    operations = [
        migrations.AddField(
            model_name="datalayer",
            name="modified_at",
            field=models.DateTimeField(auto_now=True),
        ),
        migrations.AddField(
            model_name="datalayer",
            name="share_status",
            field=models.SmallIntegerField(
                choices=[(0, "Inherit"), (99, "Deleted")],
                default=0,
                verbose_name="share status",
            ),
        ),
    ]



#!/usr/bin/env python
import os

from django.core import management


def main():
    os.environ.setdefault("DJANGO_SETTINGS_MODULE", "umap.settings")
    management.execute_from_command_line()


if __name__ == "__main__":
    main()



import re
from copy import deepcopy

import pytest
from playwright.sync_api import expect

from ..base import DataLayerFactory

pytestmark = pytest.mark.django_db

DATALAYER_DATA = {
    "type": "FeatureCollection",
    "features": [
        {
            "type": "Feature",
            "properties": {
                "name": "test marker",
                "description": "Some description",
            },
            "geometry": {
                "type": "Point",
                "coordinates": [14.6889, 48.5529, 241],
            },
        },
    ],
}


@pytest.fixture
def bootstrap(map, live_server):
    DataLayerFactory(map=map, data=DATALAYER_DATA)


def test_should_open_popup_on_click(live_server, map, page, bootstrap):
    page.goto(f"{live_server.url}{map.get_absolute_url()}")
    expect(page.locator(".umap-icon-active")).to_be_hidden()
    page.locator(".leaflet-marker-icon").click()
    expect(page.locator(".umap-icon-active")).to_be_visible()
    expect(page.locator(".leaflet-popup-content-wrapper")).to_be_visible()
    expect(page.get_by_role("heading", name="test marker")).to_be_visible()
    expect(page.get_by_text("Some description")).to_be_visible()
    # Close popup, clicking on the map, but outside of the popup.
    page.locator("#map").click(position={"x": 50, "y": 50})
    expect(page.locator(".umap-icon-active")).to_be_hidden()


def test_should_handle_locale_var_in_description(live_server, map, page):
    data = deepcopy(DATALAYER_DATA)
    data["features"][0]["properties"]["description"] = (
        "this is a link to [[https://domain.org/?locale={locale}|Wikipedia]]"
    )
    DataLayerFactory(map=map, data=data)
    page.goto(f"{live_server.url}{map.get_absolute_url()}")
    page.locator(".leaflet-marker-icon").click()
    link = page.get_by_role("link", name="Wikipedia")
    expect(link).to_be_visible()
    expect(link).to_have_attribute("href", "https://domain.org/?locale=en")


def test_should_use_custom_label_key_in_popup_default_template(live_server, map, page):
    data = deepcopy(DATALAYER_DATA)
    data["features"][0]["properties"] = {
        "libellé": "my custom label",
    }
    data["_umap_options"] = {"labelKey": "libellé"}
    DataLayerFactory(map=map, data=data)
    page.goto(f"{live_server.url}{map.get_absolute_url()}")
    page.locator(".leaflet-marker-icon").click()
    expect(page.locator(".umap-popup h4")).to_have_text("my custom label")


def test_should_display_tooltip_with_variable(live_server, map, page, bootstrap):
    map.settings["properties"]["showLabel"] = True
    map.settings["properties"]["labelKey"] = "Foo {name}"
    map.save()
    page.goto(f"{live_server.url}{map.get_absolute_url()}")
    expect(page.get_by_text("Foo test marker")).to_be_visible()


def test_should_open_popup_panel_on_click(live_server, map, page, bootstrap):
    map.settings["properties"]["popupShape"] = "Panel"
    map.save()
    page.goto(f"{live_server.url}{map.get_absolute_url()}")
    panel = page.locator(".panel.left.on")
    expect(panel).to_be_hidden()
    page.locator(".leaflet-marker-icon").click()
    expect(panel).to_be_visible()
    expect(panel).to_have_class(re.compile(".*expanded.*"))
    expect(panel.get_by_role("heading", name="test marker")).to_be_visible()
    expect(panel.get_by_text("Some description")).to_be_visible()
    # Close popup
    page.locator("#map").click()
    expect(panel).to_be_hidden()


def test_extended_properties_in_popup(live_server, map, page, bootstrap):
    map.settings["properties"]["popupContentTemplate"] = """
    Rank: {rank}
    Locale: {locale}
    Lang: {lang}
    Lat: {lat}
    Lon: {lon}
    Alt: {alt}
    Zoom: {zoom}
    Layer: {layer}
    """
    map.save()
    page.goto(f"{live_server.url}{map.get_absolute_url()}")
    expect(page.locator(".umap-icon-active")).to_be_hidden()
    page.locator(".leaflet-marker-icon").click()
    expect(page.locator(".umap-icon-active")).to_be_visible()
    expect(page.locator(".leaflet-popup-content-wrapper")).to_be_visible()
    expect(page.get_by_text("Rank: 1")).to_be_visible()
    expect(page.get_by_text("Locale: en")).to_be_visible()
    expect(page.get_by_text("Lang: en")).to_be_visible()
    expect(page.get_by_text("Lat: 48.5529")).to_be_visible()
    expect(page.get_by_text("Lon: 14.6889")).to_be_visible()
    expect(page.get_by_text("Alt: 241")).to_be_visible()
    expect(page.get_by_text("Zoom: 7")).to_be_visible()
    expect(page.get_by_text("Layer: test datalayer")).to_be_visible()


def test_only_visible_markers_are_added_to_dom(live_server, map, page):
    data = {
        "type": "FeatureCollection",
        "features": [
            {
                "type": "Feature",
                "properties": {
                    "name": "marker 1",
                    "description": "added to dom",
                },
                "geometry": {
                    "type": "Point",
                    "coordinates": [14.6, 48.5],
                },
            },
            {
                "type": "Feature",
                "properties": {
                    "name": "marker 2",
                    "description": "not added to dom at load",
                },
                "geometry": {
                    "type": "Point",
                    "coordinates": [12.6, 44.5],
                },
            },
        ],
    }
    DataLayerFactory(map=map, data=data)
    map.settings["properties"]["showLabel"] = True
    map.save()
    page.goto(f"{live_server.url}{map.get_absolute_url()}")
    markers = page.locator(".leaflet-marker-icon")
    tooltips = page.locator(".leaflet-tooltip")
    expect(markers).to_have_count(1)
    expect(tooltips).to_have_count(1)

    # Zoom in/out to show the other marker
    page.get_by_label("Zoom out").click()
    expect(markers).to_have_count(2)
    expect(tooltips).to_have_count(2)
    page.get_by_label("Zoom in").click()
    expect(markers).to_have_count(1)
    expect(tooltips).to_have_count(1)

    # Drag map to show/hide the marker
    map_el = page.locator("#map")
    map_el.drag_to(
        map_el,
        source_position={"x": 100, "y": 600},
        target_position={"x": 100, "y": 200},
    )
    expect(markers).to_have_count(2)
    expect(tooltips).to_have_count(2)
    map_el.drag_to(
        map_el,
        source_position={"x": 100, "y": 600},
        target_position={"x": 100, "y": 200},
    )
    expect(markers).to_have_count(1)
    expect(tooltips).to_have_count(1)


def test_should_display_tooltip_on_hover(live_server, map, page, bootstrap):
    map.settings["properties"]["showLabel"] = None
    map.settings["properties"]["labelKey"] = "Foo {name}"
    map.save()
    page.goto(f"{live_server.url}{map.get_absolute_url()}")
    expect(page.get_by_text("Foo test marker")).to_be_hidden()
    page.locator(".leaflet-marker-icon").hover()
    expect(page.get_by_text("Foo test marker")).to_be_visible()



import json
import os

import boto3
import pytest
from botocore.errorfactory import ClientError
from django.core.files.base import ContentFile
from django.core.files.storage import storages
from moto import mock_aws

from umap.models import DataLayer

from .base import DataLayerFactory

pytestmark = pytest.mark.django_db


@pytest.fixture(scope="module", autouse=True)
def patch_storage():
    """Mocked AWS Credentials for moto."""
    os.environ["AWS_ACCESS_KEY_ID"] = "testing"
    os.environ["AWS_SECRET_ACCESS_KEY"] = "testing"
    os.environ["AWS_SECURITY_TOKEN"] = "testing"
    os.environ["AWS_SESSION_TOKEN"] = "testing"
    os.environ["AWS_DEFAULT_REGION"] = "us-east-1"
    before = DataLayer.geojson.field.storage

    DataLayer.geojson.field.storage = storages.create_storage(
        {
            "BACKEND": "umap.storage.s3.S3DataStorage",
            "OPTIONS": {
                "access_key": "testing",
                "secret_key": "testing",
                "bucket_name": "umap",
                "region_name": "us-east-1",
            },
        }
    )
    yield
    DataLayer.geojson.field.storage = before


@pytest.fixture(scope="module", autouse=True)
def mocked_aws():
    """
    Mock all AWS interactions
    Requires you to create your own boto3 clients
    """
    with mock_aws():
        client = boto3.client("s3", region_name="us-east-1")
        client.create_bucket(Bucket="umap")
        client.put_bucket_versioning(
            Bucket="umap", VersioningConfiguration={"Status": "Enabled"}
        )
        yield


def test_can_create_datalayer(map, datalayer):
    other = DataLayerFactory(map=map)
    assert datalayer.geojson.name == f"{datalayer.pk}.geojson"
    assert other.geojson.name == f"{other.pk}.geojson"


def test_clone_should_return_new_instance(map, datalayer):
    clone = datalayer.clone()
    assert datalayer.pk != clone.pk
    assert datalayer.name == clone.name
    assert datalayer.map == clone.map
    assert datalayer.geojson != clone.geojson
    assert datalayer.geojson.name != clone.geojson.name
    assert clone.geojson.name == f"{clone.pk}.geojson"


def test_update_should_add_version(map, datalayer):
    assert len(datalayer.versions) == 1
    datalayer.geojson = ContentFile("{}", "foo.json")
    datalayer.save()
    assert len(datalayer.versions) == 2


def test_get_version(map, datalayer):
    assert len(datalayer.versions) == 1
    datalayer.geojson = ContentFile(b'{"foo": "bar"}', "foo.json")
    datalayer.save()
    assert len(datalayer.versions) == 2
    latest = datalayer.versions[0]["ref"]
    version = datalayer.get_version(latest)
    assert json.loads(version) == {"foo": "bar"}
    older = datalayer.versions[1]["ref"]
    version = datalayer.get_version(older)
    assert json.loads(version) == {
        "_umap_options": {
            "browsable": True,
            "displayOnLoad": True,
            "name": "test datalayer",
        },
        "features": [
            {
                "geometry": {
                    "coordinates": [
                        14.68896484375,
                        48.55297816440071,
                    ],
                    "type": "Point",
                },
                "id": "ExNTQ",
                "properties": {
                    "_umap_options": {
                        "color": "DarkCyan",
                        "iconClass": "Ball",
                    },
                    "description": "Da place anonymous again 755",
                    "name": "Here",
                },
                "type": "Feature",
            },
        ],
        "type": "FeatureCollection",
    }

    latest = datalayer.reference_version
    version = datalayer.get_version(latest)
    assert json.loads(version) == {"foo": "bar"}


def test_delete_datalayer_should_delete_all_versions(datalayer):
    # create a new version
    datalayer.geojson = ContentFile('{"foo": "bar"}', "foo.json")
    datalayer.save()
    s3_key = datalayer.geojson.name
    datalayer.delete()
    with pytest.raises(ClientError):
        datalayer.geojson.storage.connection.meta.client.get_object(
            Bucket="umap",
            Key=s3_key,
        )
